// DHT11 programming
// Melec.ir 
#include <mega8.h>
#include <alcd.h>
#include <delay.h>
#include <stdio.h>
#include <string.h>

#define DHT11_PORT PORTC
#define DHT11_PORTPIN PORTC0
#define DHT11_DDR DDRC
#define DHT11_DDRPIN DDC0
#define DHT11_PINPORT PINC
#define DHT11_PIN PINC0

#define DHT11_INPUT_MODE() DHT11_DDR&=~(1<<DHT11_DDRPIN)
#define DHT11_OUTPUT_MODE() DHT11_DDR|=(1<<DHT11_DDRPIN)
#define DHT11_LOW() DHT11_PORT&=~(1<<DHT11_PORTPIN)
#define DHT11_HIGH() DHT11_PORT|=(1<<DHT11_PORTPIN)

#define DHTLIB_OK                               0
#define DHTLIB_ERROR_CHECKSUM					-1
#define DHTLIB_ERROR_TIMEOUT					-2

#ifndef RXB8
#define RXB8 1
#endif

#ifndef TXB8
#define TXB8 0
#endif

#ifndef UPE
#define UPE 2
#endif

#ifndef DOR
#define DOR 3
#endif

#ifndef FE
#define FE 4
#endif

#ifndef UDRE
#define UDRE 5
#endif

#ifndef RXC
#define RXC 7
#endif

#define FRAMING_ERROR (1<<FE)
#define PARITY_ERROR (1<<UPE)
#define DATA_OVERRUN (1<<DOR)
#define DATA_REGISTER_EMPTY (1<<UDRE)
#define RX_COMPLETE (1<<RXC)
//end of usart defines

int read_dht11(int *temp,int *humidity);
void main(void)
{
char lcd_buff[17];
int temp=0;
int humidity=0;
int status=0;


lcd_init(16);
lcd_clear();
lcd_puts("Melec.ir >>DHT11");
delay_ms(600);


 
#asm("sei")
while (1)
      {    
      
      status=read_dht11(&temp,&humidity);
      sprintf(lcd_buff,"Temp:%02d H:%02d ",temp,humidity);      
      lcd_clear();   
      lcd_puts(lcd_buff);
      delay_ms(1);
      lcd_gotoxy(0,1);
      lcd_putsf("Melec.ir");
      delay_ms(2000); //dht11 needs 2000ms ...      
                 
      }
}


// read_dht11 functions
int read_dht11(int *temp,int *humidity){
unsigned char bytes[5];
unsigned char sum;
unsigned int bitcount=7;
unsigned int loopcount=0;
unsigned int byteid=0;
int i=0;
// EMPTY BUFFER
	for (i=0; i< 5; i++) bytes[i] = 0;
// REQUEST SAMPLE
	DHT11_OUTPUT_MODE();  
    DHT11_LOW();    
    delay_ms(100);
    DHT11_HIGH();
	delay_us(25);
	DHT11_INPUT_MODE();   
    DHT11_LOW();
    loopcount=0;    
	while(!(DHT11_PINPORT&(1<<DHT11_PIN))){ //pin is low
     delay_us(1);
     loopcount++;  
    }	  
    if (loopcount>85) return DHTLIB_ERROR_TIMEOUT;
    loopcount=0;    
	while((DHT11_PINPORT&(1<<DHT11_PIN))){ //pin is high
     delay_us(1);
     loopcount++;
    }     
     if (loopcount>85) return DHTLIB_ERROR_TIMEOUT;
    for(i=0;i<40;i++){ 
    loopcount=0;    
    while(!(DHT11_PINPORT&(1<<DHT11_PIN))){}  
    while((DHT11_PINPORT&(1<<DHT11_PIN)) && loopcount<100){loopcount++;delay_us(1);}
    if(loopcount>15)
    bytes[byteid] |= (1 << bitcount);
     if (bitcount == 0)   // next byte?
		{
			bitcount = 7;    // restart at MSB
			byteid++;      // next byte!
		}
		else bitcount--; 
    } 
    *humidity    = bytes[0];
	*temp = bytes[2];
    sum = bytes[0] + bytes[2];  
	if (bytes[4] != sum) return DHTLIB_ERROR_CHECKSUM;  
    return DHTLIB_OK;
}